﻿Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices

Public Class Bitmap32
    ' Provide public access to the picture's byte data.
    Public ImageBytes() As Byte
    Public RowSizeBytes As Integer
    Public Const PixelDataSize As Integer = 32

    ' A reference to the Bitmap.
    Private m_Bitmap As Bitmap

    ' Save a reference to the bitmap.
    Public Sub New(ByVal bm As Bitmap)
        m_Bitmap = bm
    End Sub

    ' Bitmap data.
    Private m_BitmapData As BitmapData

    ' Return the image's dimensions.
    Public ReadOnly Property Width() As Integer
        Get
            Return m_Bitmap.Width
        End Get
    End Property
    Public ReadOnly Property Height() As Integer
        Get
            Return m_Bitmap.Height
        End Get
    End Property

    ' Provide easy access to the color values.
    Public Sub GetPixel(ByVal x As Integer, ByVal y As Integer, ByRef red As Byte, ByRef green As Byte, ByRef blue As Byte, ByRef alpha As Byte)
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        blue = ImageBytes(i)
        i += 1
        green = ImageBytes(i)
        i += 1
        red = ImageBytes(i)
        i += 1
        alpha = ImageBytes(i)
    End Sub
    Public Sub SetPixel(ByVal x As Integer, ByVal y As Integer, ByVal red As Byte, ByVal green As Byte, ByVal blue As Byte, ByVal alpha As Byte)
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        ImageBytes(i) = blue
        i += 1
        ImageBytes(i) = green
        i += 1
        ImageBytes(i) = red
        i += 1
        ImageBytes(i) = alpha
    End Sub
    Public Function GetBlue(ByVal x As Integer, ByVal y As Integer) As Byte
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        Return ImageBytes(i)
    End Function
    Public Sub SetBlue(ByVal x As Integer, ByVal y As Integer, ByVal blue As Byte)
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        ImageBytes(i) = blue
    End Sub
    Public Function GetGreen(ByVal x As Integer, ByVal y As Integer) As Byte
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        Return ImageBytes(i + 1)
    End Function
    Public Sub SetGreen(ByVal x As Integer, ByVal y As Integer, ByVal green As Byte)
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        ImageBytes(i + 1) = green
    End Sub
    Public Function GetRed(ByVal x As Integer, ByVal y As Integer) As Byte
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        Return ImageBytes(i + 2)
    End Function
    Public Sub SetRed(ByVal x As Integer, ByVal y As Integer, ByVal red As Byte)
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        ImageBytes(i + 2) = red
    End Sub
    Public Function GetAlpha(ByVal x As Integer, ByVal y As Integer) As Byte
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        Return ImageBytes(i + 3)
    End Function
    Public Sub SetAlpha(ByVal x As Integer, ByVal y As Integer, ByVal alpha As Byte)
        Dim i As Integer = y * m_BitmapData.Stride + x * 4
        ImageBytes(i + 3) = alpha
    End Sub

    ' Lock the bitmap's data.
    Public Sub LockBitmap()
        ' Lock the bitmap data.
        Dim bounds As New Rectangle( _
            0, 0, m_Bitmap.Width, m_Bitmap.Height)
        m_BitmapData = m_Bitmap.LockBits(bounds, _
            ImageLockMode.ReadWrite, _
            PixelFormat.Format32bppArgb)
        RowSizeBytes = m_BitmapData.Stride

        ' Allocate room for the data.
        Dim total_size As Integer = m_BitmapData.Stride * m_BitmapData.Height
        ReDim ImageBytes(0 To total_size - 1)

        ' Copy the data into the ImageBytes array.
        Marshal.Copy(m_BitmapData.Scan0, ImageBytes, 0, total_size)
    End Sub

    ' Copy the data back into the Bitmap
    ' and release resources.
    Public Sub UnlockBitmap()
        ' Copy the data back into the bitmap.
        Dim total_size As Integer = m_BitmapData.Stride * m_BitmapData.Height
        Marshal.Copy(ImageBytes, 0, m_BitmapData.Scan0, total_size)

        ' Unlock the bitmap.
        m_Bitmap.UnlockBits(m_BitmapData)

        ' Release resources.
        ImageBytes = Nothing
        m_BitmapData = Nothing
    End Sub
End Class
