﻿Imports System.Drawing.Imaging
Imports System.IO

Public Class Form1
    ' Let the user select a file.
    Private Sub mnuFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileOpen.Click
        If (ofdOriginal.ShowDialog() = Windows.Forms.DialogResult.OK) Then
            Dim margin As Integer = picInput.Left
            picInput.Image = Bitmap.FromFile(ofdOriginal.FileName)
            picOutput.Image = Nothing
            picOutput.Left = picInput.Right + Margin
            picOutput.ClientSize = picInput.ClientSize
            Me.ClientSize = New Size( _
                picOutput.Right + margin, _
                picOutput.Bottom + margin)
        End If
    End Sub

    ' Save the result image.
    Private Sub mnuFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileSave.Click
        If (sfdResult.ShowDialog() = Windows.Forms.DialogResult.OK) Then
            Dim bm As Bitmap = DirectCast(picOutput.Image, Bitmap)
            SaveBitmapUsingExtension(bm, sfdResult.FileName)
        End If
    End Sub

    ' Save the file with the appropriate format.
    ' Throw a NotSupportedException if the file
    ' has an unknown extension.
    Public Sub SaveBitmapUsingExtension(ByVal bm As Bitmap, ByVal filename As String)
        Dim extension As String = Path.GetExtension(filename)
        Select extension.ToLower()
            Case ".bmp"
                bm.Save(filename, ImageFormat.Bmp)
            Case ".exif"
                bm.Save(filename, ImageFormat.Exif)
            Case ".gif"
                bm.Save(filename, ImageFormat.Gif)
            Case ".jpg", ".jpeg"
                bm.Save(filename, ImageFormat.Jpeg)
            Case ".png"
                bm.Save(filename, ImageFormat.Png)
            Case ".tif", ".tiff"
                bm.Save(filename, ImageFormat.Tiff)
            Case Else
                Throw New NotSupportedException( _
                    "Unknown file extension " & extension)
        End Select
    End Sub

    ' Standard color sets.
    Private Sub mnuColorsSet1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuColorsSet1.Click
        picFromColor0.BackColor = Color.Red
        picFromColor1.BackColor = Color.Green
        picFromColor2.BackColor = Color.Blue
        picFromColor3.BackColor = Color.Black
        picFromColor4.BackColor = Color.White
        picToColor0.BackColor = Color.Red
        picToColor1.BackColor = Color.Green
        picToColor2.BackColor = Color.Blue
        picToColor3.BackColor = Color.Black
        picToColor4.BackColor = Color.White
    End Sub

    Private Sub mnuColorsSet2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuColorsSet2.Click
        picFromColor0.BackColor = Color.Red
        picFromColor1.BackColor = Color.Green
        picFromColor2.BackColor = Color.Blue
        picFromColor3.BackColor = Color.Black
        picFromColor4.BackColor = Color.White
        picToColor0.BackColor = Color.Yellow
        picToColor1.BackColor = Color.Orange
        picToColor2.BackColor = Color.Purple
        picToColor3.BackColor = Color.Violet
        picToColor4.BackColor = Color.Lime
    End Sub

    Private Sub mnuColorsSet3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuColorsSet3.Click
        picFromColor0.BackColor = Color.Pink
        picFromColor1.BackColor = Color.LightGreen
        picFromColor2.BackColor = Color.LightBlue
        picFromColor3.BackColor = Color.Gray
        picFromColor4.BackColor = Color.White
        picToColor0.BackColor = Color.Pink
        picToColor1.BackColor = Color.LightGreen
        picToColor2.BackColor = Color.LightBlue
        picToColor3.BackColor = Color.Gray
        picToColor4.BackColor = Color.White
    End Sub

    Private Sub mnuColorsSet4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuColorsSet4.Click
        picFromColor0.BackColor = Color.Red
        picFromColor1.BackColor = Color.Green
        picFromColor2.BackColor = Color.Blue
        picFromColor3.BackColor = Color.Black
        picFromColor4.BackColor = Color.White
        picToColor0.BackColor = Color.Pink
        picToColor1.BackColor = Color.LightGreen
        picToColor2.BackColor = Color.LightBlue
        picToColor3.BackColor = Color.Gray
        picToColor4.BackColor = Color.Khaki
    End Sub

    ' Warholize.
    Private Sub btnGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGo.Click
        picOutput.Image = Nothing
        picOutput.Refresh()

        ' Get the input and output color data.
        Dim in_boxes() As PictureBox = {picFromColor0, picFromColor1, picFromColor2, picFromColor3, picFromColor4}
        Dim out_boxes() As PictureBox = {picToColor0, picToColor1, picToColor2, picToColor3, picToColor4}
        Dim in_r(0 To in_boxes.Length - 1) As Integer
        Dim in_g(0 To in_boxes.Length - 1) As Integer
        Dim in_b(0 To in_boxes.Length - 1) As Integer
        Dim out_r(0 To in_boxes.Length - 1) As Byte
        Dim out_g(0 To in_boxes.Length - 1) As Byte
        Dim out_b(0 To in_boxes.Length - 1) As Byte
        For i As Integer = 0 To in_boxes.Length - 1
            in_r(i) = in_boxes(i).BackColor.R
            in_g(i) = in_boxes(i).BackColor.G
            in_b(i) = in_boxes(i).BackColor.B
            out_r(i) = out_boxes(i).BackColor.R
            out_g(i) = out_boxes(i).BackColor.G
            out_b(i) = out_boxes(i).BackColor.B
        Next i

        ' Get and lock the Bitmap32.
        Dim original_bm As Bitmap = DirectCast(picInput.Image, Bitmap)
        Dim bm As New Bitmap(original_bm)
        Dim bm32 As New Bitmap32(bm)
        bm32.LockBitmap()

        ' Process the pixels.
        For y As Integer = 0 To bm.Height - 1
            For x As Integer = 0 To bm.Width - 1
                ' Process pixel (row, col).
                Dim r, g, b, a As Byte
                bm32.GetPixel(x, y, r, g, b, a)
                Dim best_i As Integer = 0
                Dim best_dist As Integer = Integer.MaxValue
                For i As Integer = 0 To in_boxes.Length - 1
                    ' Compute the distance from this pixel to input pixel i.
                    Dim dr As Integer = r - in_r(i)
                    Dim dg As Integer = g - in_g(i)
                    Dim db As Integer = b - in_b(i)
                    Dim dist As Integer = dr * dr + dg * dg + db * db

                    ' See if this is an improvement.
                    If (dist < best_dist) Then
                        best_dist = dist
                        best_i = i
                    End If
                Next i

                ' Update the pixel.
                bm32.SetPixel(x, y, out_r(best_i), out_g(best_i), out_b(best_i), 255)
            Next x
        Next y

        ' Unlock the Bitmap32.
        bm32.UnlockBitmap()

        ' Display the result.
        picOutput.Image = bm
    End Sub

    ' Let the user pick this color.
    Private Sub picColor_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles picFromColor0.Click, picToColor4.Click, picToColor3.Click, picToColor2.Click, picToColor1.Click, picToColor0.Click, picFromColor4.Click, picFromColor3.Click, picFromColor2.Click, picFromColor1.Click
        Dim pic As PictureBox = DirectCast(sender, PictureBox)
        cdColor.Color = pic.BackColor
        If (cdColor.ShowDialog() = DialogResult.OK) Then
            pic.BackColor = cdColor.Color
        End If
    End Sub
End Class
